<?php
/**
 * Micropayment-Script to verify that SSL/TLS connections work correctly
 * Release-Date: 2015-10-21
 */

/* Host config
 ******************************************************************************************************************** */
$host = 'sslcheck.micropayment.de';
$port = 443;
$path = '/';

/* stream_socket_client
 ******************************************************************************************************************** */
printTitle('stream_socket_client', 'MCP-Service-Client (API) v1.21+');

outputBegin();
$errorNo = 0;
$errorMsg = '';

// @formatter:off
$opts = [
    'ssl' => [
        'verify_peer'       => true,
        'verify_peer_name'  => true,
        'allow_self_signed' => true,
        //'capture_session_meta' => true,
        'crypto_method'       => STREAM_CRYPTO_METHOD_TLSv1_0_CLIENT | STREAM_CRYPTO_METHOD_TLSv1_1_CLIENT | STREAM_CRYPTO_METHOD_TLSv1_2_CLIENT,
    ]
];
// @formatter:on
$context = stream_context_create($opts);
$fp = stream_socket_client(sprintf('%s://%s:%d', 'tls', $host, $port), $errorNo, $errorMsg, 30, STREAM_CLIENT_CONNECT, $context);
if (!$fp) {
    printError(sprintf('An Error occurred during connect to `%s:%d` #%d `%s`', $host, $port, $errorNo, $errorMsg));
}
else {
    fwrite($fp, sprintf("%s %s HTTP/1.1\n", 'GET', $path));
    fwrite($fp, sprintf("Host: %s \n", $host));
    fwrite($fp, "Connection: close\n\n");

    while (!feof($fp)) {
        echo fread($fp, 128);
    }
    @fclose($fp);
}
outputEnd();

printSeparator();


/* fsockopen: MCP-Service-Client (API) <v1.21
 ******************************************************************************************************************** */
printTitle('fsockopen', 'MCP-Service-Client (API) <v1.21');

outputBegin();
$errorNo     = 0;
$errorMsg    = '';
$fp = @fsockopen('ssl://' . $host, $port, $errorNo, $errorMsg, 30);
if(!$fp) {
    printError(sprintf('An Error occurred during connect to `%s:%d` #%d `%s`', $host, $port, $errorNo, $errorMsg));
}
else {
    fwrite($fp, sprintf("%s %s HTTP/1.1\n", 'GET', $path) );
    fwrite($fp, sprintf("Host: %s \n", $host) );
    fwrite($fp, "Connection: close\n\n");

    while(!feof($fp)) {
        echo fread($fp, 128);
    }
    @fclose($fp);
}
outputEnd();

printSeparator();



/* CURL(PHP): manual / other http clients
 ******************************************************************************************************************** */
printTitle('CURL(PHP)', 'manual / other http clients');

if(function_exists('curl_init')) {
    $ch = curl_init();
    curl_setopt($ch, CURLOPT_URL, 'https://' . $host . $path);
    curl_setopt($ch, CURLOPT_HEADER, true);
    curl_setopt($ch, CURLOPT_SSLVERSION, 1);
    //curl_setopt($ch, CURLOPT_SSL_CIPHER_LIST, 'TLSv1');
    curl_setopt($ch, CURLOPT_SSL_CIPHER_LIST, 'ECDHE-RSA-AES128-GCM-SHA256 ECDHE-RSA-AES128-SHA256 ECDHE-RSA-AES128-SHA ECDHE-RSA-AES128-GCM-SHA256 ECDHE-RSA-AES128-SHA256 ECDHE-RSA-AES128-SHA');

    outputBegin();
    curl_exec($ch);

    if (($errorNo = curl_errno($ch)) !== 0) {
        $errorMsg = curl_error($ch);
        printError(sprintf('An Error occurred during connect to `%s:%d` #%d `%s`', $host, $port, $errorNo, $errorMsg));
    }
    curl_close($ch);
    outputEnd();
}
else {
    printError('CURL module not exists');
}


printSeparator();


/* Utils
 ******************************************************************************************************************** */
function printTitle($main, $sub)
{
    if(PHP_SAPI === 'cli')
    {
        cliPrint('title', PHP_EOL . $main . ' - ' . $sub . PHP_EOL, true);
        cliPrint('title', str_repeat('-', 80) . PHP_EOL, true);
        return;
    }

    echo '<h1>' . htmlentities($main) .' <small style="color: darkgrey">' . htmlentities($sub) . '</small></h1>';
}

function printSeparator()
{
    if(PHP_SAPI === 'cli') {
        echo str_repeat('=', 80) . PHP_EOL;
        return;
    }
    echo '<hr>';
}

function printError($message)
{
    if(PHP_SAPI === 'cli'){
        cliPrint('error', $message . PHP_EOL, true);
        return;
    }

    echo '<b style="color: red">' . htmlentities($message) . '</b>';
}

function outputBegin()
{
    if(PHP_SAPI === 'cli'){
        cliPrint('green', PHP_EOL, false);
        return;
    }

    echo '<pre style="color: darkgreen">';
}

function outputEnd()
{
    if(PHP_SAPI === 'cli'){
        cliPrint('green', PHP_EOL, true);
        return;
    }

    echo '</pre>';
}


function cliPrint($style, $message, $reset=true)
{
    // @formatter:off
    $styles = [
        'reset'   => "\033[0m",
        'red'     => "\033[31m",
        'green'   => "\033[32m",
        'info'    => "\033[36m",
        'error'   => "\033[37;41m",
        'success' => "\033[37;42m",
        'title'   => "\033[33m",
    ];
    // @formatter:on
    static $isAnsiCon = null;

    if (null === $isAnsiCon) {
        if (DIRECTORY_SEPARATOR === '\\') {
            $isAnsiCon = false !== getenv('ANSICON') || 'ON' === getenv('ConEmuANSI');
        }
        else {
            $isAnsiCon = function_exists('posix_isatty') && @posix_isatty(STDOUT);
        }
    }
    echo ($isAnsiCon && isset($styles[$style]) ? $styles[$style] : '') . $message . ($isAnsiCon && $reset ? $styles['reset'] : '');
}
